#!/bin/sh
#
# Test of checkpoint/restart of an expect script.
# Note that the outer shell script is bourne shell

# Check for expect
expect -c exit || exit 77

top_builddir=${top_builddir:-..}
export top_builddir
\rm -f Context[123] .Context[123].tmp 2>/dev/null
sh <<-'__OUTER__'
	exec 2>/dev/null # Drop job control message(s) from the outer script
	$top_builddir/util/cr_run/cr_run expect 2>&1 <<-'__INNER__'
		set top_blddir $env(top_builddir)
		set chkpt_cmd "$top_blddir/util/cr_checkpoint/cr_checkpoint"
		set rstrt_cmd "$top_blddir/util/cr_restart/cr_restart"
		proc waitfor {pid} { while { ! [ catch { wait ; exec kill -s 0 $pid } msg ] } { sleep 1 } }
		proc checkpoint {file child} {
		    global chkpt_cmd
		    system "$chkpt_cmd --file=$file --tree --term $child"
		    if { $child != [ pid ] } { waitfor $child }
		    sleep 1
		}
		proc restart {file} {
		    global rstrt_cmd
		    set child [ exec -- $rstrt_cmd $file & ]
		    sleep 3
		    return $child
		}
		proc echo {msg} { puts $msg; flush stdout }
		flush stderr
		flush stdout
		set child [ fork ]
		if { $child == 0 } {
		    for { set i 0 } { $i < 16 } { incr i } {
			puts "$i Hello"
			flush stdout
			sleep 1
		    }
		    exit
		}
		sleep 3
		echo "# Checkpoint original child"
		checkpoint "Context1" $child
		echo "# Restart 1"
		set $child [ restart "Context1" ]
		echo "# Checkpoint restarted child"
		checkpoint "Context2" $child
		echo "# Restart^2"
		set $child [ restart "Context2" ]
		echo "# Checkpoint self"
		checkpoint "Context3" [ pid ]
		waitfor $child
		echo "16 DONE"
		__INNER__
	exec 2>&1
	sleep 3
	echo "# Restart interpreter"
	$top_builddir/util/cr_restart/cr_restart Context3
	__OUTER__
\rm -f Context[123] .Context[123].tmp 2>/dev/null
